package com.winhc.widget;

import android.content.Context;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v7.app.AppCompatActivity;
import android.view.WindowManager;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;

import com.winhc.widget.util.ActivityManageUtil;
import com.winhc.widget.util.StatusBarUtils;

import java.util.List;

/**
 * Create by LZ at 2021/10/12
 */
public abstract class BaseActivity<T extends IPresenter> extends AppCompatActivity
        implements IBaseView, EasyPermissions.PermissionCallbacks {
    protected T mPresenter;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        defaultAnimator();
        beforeCreate();
        setContentView(initContentView());
        mPresenter = initPresenter();
        if (mPresenter != null) {
            mPresenter.attachView(this);
        }
        initSystemBar();
        initView();
        initData(savedInstanceState);
        //保存到窗口栈
        ActivityManageUtil
                .getInstance()
                .pushToStatic(this.getClass(), ActivityManageUtil.ACTIVITY_START_TYPE.Standard)
                .pushActivityObj(this);
    }

    /**
     * 处理Window什么东西
     */
    protected abstract void beforeCreate();

    /**
     * 默认的进入界面的动画
     */
    protected void defaultAnimator() {
        overridePendingTransition(R.anim.slide_in_right, R.anim.anim_no);
    }

    protected int initContentView() {
        return R.layout.activity_base;
    }

    public abstract T initPresenter();

    /**
     * 初始化视图控件
     */
    protected abstract void initView();

    protected void initData(Bundle savedInstanceState) {
    }

    protected void initSystemBar() {
        StatusBarUtils.setFullWindow(this);
        StatusBarUtils.setStatusBarColorBlack(this);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (mPresenter != null) {
            mPresenter.detachView();
        }
        ActivityManageUtil
                .getInstance()
                .finishActivity(this)
                .quitActivityObj(this, false);
    }

    @Override
    public void finish() {
        super.finish();
        overridePendingTransition(R.anim.anim_no, R.anim.slide_out_right);
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        EasyPermissions.onRequestPermissionsResult(requestCode, permissions, grantResults, this);
    }

    @Override
    public void onPermissionsAllGranted() {

    }

    @Override
    public void onPermissionsGranted(int requestCode, List<String> perms) {

    }

    @Override
    public void onPermissionsDenied(int requestCode, List<String> perms) {
        EasyPermissions.checkDeniedPermissionsNeverAskAgain(this,
                "当前应用缺少必要权限，\n请点击\"设置\"-\"权限\"-打开所需权限，\n然后点击后退按钮，即可返回。",
                "去设置", "取消", null, perms);
    }


    @Override
    public void showLoadingView() {

    }

    @Override
    public void showNetWorkError() {

    }

    @Override
    public void showNetWorkError(Throwable t, String errorCode) {

    }

    @Override
    public void showToast() {

    }

    //是否显示软键盘
    protected void showKeyboard(boolean isShow) {
        InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
        if (isShow) {
            if (getCurrentFocus() == null) {
                imm.toggleSoftInput(InputMethodManager.SHOW_IMPLICIT, 0);
            } else {
                imm.showSoftInput(getCurrentFocus(), 0);
            }
        } else {
            if (getCurrentFocus() != null) {
                imm.hideSoftInputFromWindow(getCurrentFocus().getWindowToken(), 0);
            }
        }
    }

    protected void showSoftInputFromWindow(EditText editText) {
        editText.setFocusable(true);
        editText.setFocusableInTouchMode(true);
        editText.requestFocus();
        getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_VISIBLE);
    }

    protected void hideSoftInputFromWindow(EditText editText) {
        editText.setFocusable(false);
        editText.setFocusableInTouchMode(true);
        getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_HIDDEN);
    }
}
