package com.winhc.widget;

import android.app.Dialog;
import android.content.Context;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.support.annotation.IdRes;
import android.support.annotation.StyleRes;
import android.view.Gravity;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;

/**
 * Create by LZ at 2021/10/8
 * 用于辅助构建Dialog
 */
public class DialogBuilder {
    protected Dialog mDialog;
    protected Context mContext;
    protected View mContentView;

    public DialogBuilder(Context context) {
        this.mContext = context;
    }

    public DialogBuilder build(View view, @StyleRes int animId) {
        return build(view, Gravity.CENTER, animId);
    }

    public DialogBuilder buildBottom(View view, @StyleRes int animId) {
        return build(view, Gravity.BOTTOM, animId);
    }

    public DialogBuilder build(View view, int gravity, @StyleRes int animId) {
        if (isShowing()) {
            cancel();
        }
        this.mContentView = view;
        mDialog = new Dialog(mContext);
        mDialog.setCancelable(false);
        mDialog.setCanceledOnTouchOutside(true);
        mDialog.setContentView(view);
        Window window = mDialog.getWindow();
        if (window != null) {
            window.getDecorView().setPadding(0, 0, 0, 0);
            WindowManager.LayoutParams lp = window.getAttributes();
            lp.gravity = gravity;
            lp.width = WindowManager.LayoutParams.MATCH_PARENT;
            window.setWindowAnimations(animId);
            window.setAttributes(lp);
            window.setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
        }
        return this;
    }

    public DialogBuilder outsideCanceled(boolean outsideCanceled) {
        if (mDialog != null) {
            mDialog.setCanceledOnTouchOutside(outsideCanceled);
        }
        return this;
    }

    public DialogBuilder cancelable(boolean cancelable) {
        if (mDialog != null) {
            mDialog.setCancelable(cancelable);
        }
        return this;
    }

    /**
     * 根据 id 查找 View
     */
    public <V extends View> V findViewById(@IdRes int id) {
        if (mContentView == null) {
            throw new IllegalStateException("are you ok?");
        }
        return mContentView.findViewById(id);
    }

    public void show() {
        if (isShowing()) return;
        if (mDialog != null) {
            mDialog.show();
        }
    }

    public boolean isShowing() {
        if (mDialog != null) return mDialog.isShowing();
        return false;
    }

    public void cancel() {
        if (mDialog != null && mDialog.isShowing()) {
            mDialog.dismiss();
            mDialog.cancel();
            mDialog = null;
        }
    }
}
