package com.winhc.widget.util;

import android.util.Log;

/**
 * Created by toby on 2017/4/25.
 */
public class MMLog {
    private static final String DEFAULT_TAG = "develop";
    /**
     * 是否开启debug
     */
    private static boolean isDebug = false;

    public static void isDebug(boolean value) {
        isDebug = value;
    }

    /**
     * 错误
     *
     * @param tag
     * @param msg
     */
    public static void e(String tag, Object msg) {
        if (isDebug) {
            Log.e(tag, getFunctionName() + " msg : " + (msg == null ? "null object" : msg.toString()));
        }
    }

    /**
     * 错误
     *
     * @param msg
     */
    public static void e(Object msg) {
        if (isDebug) {
            Log.e(DEFAULT_TAG, getFunctionName() + " msg : " + (msg == null ? "null object" : msg.toString()));
        }
    }

    /**
     * 信息
     *
     * @param tag
     * @param msg
     */
    public static void i(String tag, Object msg) {
        if (isDebug) {
            Log.i(tag, getFunctionName() + " msg : " + (msg == null ? "null object" : msg.toString()));
        }
    }

    /**
     * 信息
     *
     * @param msg
     */
    public static void i(Object msg) {
        if (isDebug) {
            //因为String的length是字符数量不是字节数量所以为了防止中文字符过多，
            //  把4*1024的MAX字节打印长度改为2001字符数
            int max_str_length = 2001 - DEFAULT_TAG.length();
            //大于4000时
            while (msg.toString().length() > max_str_length) {
                Log.i(DEFAULT_TAG, msg.toString().substring(0, max_str_length));
                msg = msg.toString().substring(max_str_length);
            }
            //剩余部分
            Log.i(DEFAULT_TAG, msg.toString());
        }
    }

    /**
     * 警告
     *
     * @param tag
     * @param msg
     */
    public static void w(String tag, Object msg) {
        if (isDebug) {
            Log.w(tag, getFunctionName() + " msg : " + (msg == null ? "null object" : msg.toString()));
        }
    }

    /**
     * 警告
     *
     * @param msg
     */
    public static void w(Object msg) {
        if (isDebug) {
            Log.w(DEFAULT_TAG, getFunctionName() + " msg : " + (msg == null ? "null object" : msg.toString()));
        }
    }

    /**
     * 获取当前方法的文件名，方法名，行号
     *
     * @return
     */
    private static String getFunctionName() {
        StackTraceElement[] sts = Thread.currentThread().getStackTrace();
        if (sts == null) {
            return null;
        }
        for (StackTraceElement st : sts) {
            if (st.isNativeMethod()) {
                continue;
            }
            if (st.getClassName().equals(Thread.class.getName())) {
                continue;
            }
            if (st.getClassName().equals(MMLog.class.getName())) {
                continue;
            }
            return "[ " + Thread.currentThread().getId() + ": "
                    + st.getFileName() + ": " + st.getMethodName() + "(): " + st.getLineNumber() + " ]";
        }
        return null;
    }

}
