package com.winhc.widget.util;

import android.content.Context;
import android.content.SharedPreferences;

/**
 * SP 工具类
 */
public class SpUtil {
    private static SharedPreferences sharedPreferences;

    public SpUtil(Context context, String name) {
        sharedPreferences = context.getSharedPreferences(name, Context.MODE_PRIVATE);
    }

    /**
     * 保存数据
     *
     * @param key
     * @param value
     */
    public void put(String key, Object value) {
        put(key, value, false);
    }

    /**
     * 保存数据
     *
     * @param key
     * @param value
     * @param isCommit
     */
    public void put(String key, Object value, boolean isCommit) {
        SharedPreferences.Editor edit = sharedPreferences.edit();
        if (isCommit) {
            if (value instanceof String) {
                edit.putString(key, (String) value).apply();
            } else if (value instanceof Integer) {
                edit.putInt(key, (Integer) value).commit();
            } else if (value instanceof Float) {
                edit.putFloat(key, (Float) value).commit();
            } else if (value instanceof Long) {
                edit.putLong(key, (Long) value).commit();
            } else if (value instanceof Boolean) {
                edit.putBoolean(key, (Boolean) value).commit();
            }
        } else {
            if (value instanceof String) {
                edit.putString(key, (String) value).apply();
            } else if (value instanceof Integer) {
                edit.putInt(key, (Integer) value).apply();
            } else if (value instanceof Float) {
                edit.putFloat(key, (Float) value).apply();
            } else if (value instanceof Long) {
                edit.putLong(key, (Long) value).apply();
            } else if (value instanceof Boolean) {
                edit.putBoolean(key, (Boolean) value).apply();
            }
        }
    }

    public String getString(String key, String defaultValue) {
        return sharedPreferences.getString(key, defaultValue);
    }

    public  boolean getBoolean(String key, boolean defaultValue) {
        return sharedPreferences.getBoolean(key, defaultValue);
    }

    public  int getInt(String key, int defaultValue) {
        return sharedPreferences.getInt(key, defaultValue);
    }

    public  long getLong(String key, long defaultValue) {
        return sharedPreferences.getLong(key, defaultValue);
    }

    public  void remove(String... keys) {
        SharedPreferences.Editor editor = sharedPreferences.edit();
        if (null != keys && keys.length > 0) {
            for (String key : keys) {
                editor.remove(key).apply();
            }
            editor.commit();
        }
    }

    /**
     * 清除所有数据
     */
    public  void clear() {
        sharedPreferences.edit().clear().apply();
    }

    public  boolean contains(String key) {
        return sharedPreferences.contains(key);
    }
}
