package com.winhc.widget.util;

import android.content.Context;
import android.content.pm.PackageManager;
import android.os.Environment;

import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;

/**
 * 存储的路径
 */
public class StorageUtil {
    private static final String EXTERNAL_STORAGE_PERMISSION = "android.permission.WRITE_EXTERNAL_STORAGE";
    private static final String INDIVIDUAL_DIR_NAME = "Cache";

    public static String getTotalCacheSize(Context context) {
        try {
            long folderSize = getFolderSize(context.getCacheDir());
            if (Environment.getExternalStorageState().equals("mounted")) {
                folderSize += getFolderSize(context.getExternalCacheDir());
            }
            return getFormatSize((double) folderSize);
        } catch (Exception unused) {
            return "";
        }
    }

    public static long getFolderSize(File file) throws Exception {
        long j;
        long j2 = 0;
        try {
            File[] listFiles = file.listFiles();
            for (int i = 0; i < listFiles.length; i++) {
                if (listFiles[i].isDirectory()) {
                    j = getFolderSize(listFiles[i]);
                } else {
                    j = listFiles[i].length();
                }
                j2 += j;
            }
        } catch (Exception unused) {
        }
        return j2;
    }

    public static String getFormatSize(double d) {
        double d2 = d / 1024.0d;
        if (d2 < 1.0d) {
            return "";
        }
        double d3 = d2 / 1024.0d;
        if (d3 < 1.0d) {
            BigDecimal bigDecimal = new BigDecimal(Double.toString(d2));
            return bigDecimal.setScale(2, 4).toPlainString() + "KB";
        }
        double d4 = d3 / 1024.0d;
        if (d4 < 1.0d) {
            BigDecimal bigDecimal2 = new BigDecimal(Double.toString(d3));
            return bigDecimal2.setScale(2, 4).toPlainString() + "MB";
        }
        double d5 = d4 / 1024.0d;
        if (d5 < 1.0d) {
            BigDecimal bigDecimal3 = new BigDecimal(Double.toString(d4));
            return bigDecimal3.setScale(2, 4).toPlainString() + "GB";
        }
        BigDecimal bigDecimal4 = new BigDecimal(d5);
        return bigDecimal4.setScale(2, 4).toPlainString() + "TB";
    }

    public static void cleanInternalCache(Context context) {
        FileUtil.deleteFilesByDirectory(context.getCacheDir());
    }

    public static void cleanDatabases(Context context) {
        FileUtil.deleteFilesByDirectory(new File("/data/data/" + context.getPackageName() + "/databases"));
    }

    public static void cleanSharedPreference(Context context) {
        FileUtil.deleteFilesByDirectory(new File("/data/data/" + context.getPackageName() + "/shared_prefs"));
    }

    public static void cleanDatabaseByName(Context context, String str) {
        context.deleteDatabase(str);
    }

    public static void cleanFiles(Context context) {
        FileUtil.deleteFilesByDirectory(context.getFilesDir());
    }

    public static void cleanExternalCache(Context context) {
        if (Environment.getExternalStorageState().equals("mounted")) {
            FileUtil.deleteFilesByDirectory(context.getExternalCacheDir());
        }
    }

    public static void cleanExternalCacheFile(Context context, String str) {
        if (Environment.getExternalStorageState().equals("mounted")) {
            FileUtil.deleteFilesByDirectory(context.getExternalCacheDir(), str);
        }
    }

    public static void cleanCustomCache(String str) {
        FileUtil.deleteFilesByDirectory(new File(str));
    }

    public static void cleanApplicationData(Context context, String... strArr) {
        cleanInternalCache(context);
        cleanExternalCache(context);
        cleanDatabases(context);
        cleanSharedPreference(context);
        cleanFiles(context);
        for (String str : strArr) {
            cleanCustomCache(str);
        }
    }

    public static String getRootPath() {
        return Environment.getExternalStorageDirectory().getPath() + File.separator;
    }

    public static File getCacheDirectory(Context context) {
        return getCacheDirectory(context, INDIVIDUAL_DIR_NAME);
    }

    public static File getCacheDirectory(Context context, String cacheDir) {
        File appCacheDir = null;
        if (isAvailable() && hasExternalStoragePermission(context)) {
            appCacheDir = getExternalCacheDir(context, cacheDir);
        }
        if (appCacheDir == null) {
            appCacheDir = new File("/data/data/" + context.getPackageName() + File.separator + cacheDir + File.separator);
        }
        return appCacheDir;
    }

    public static File getIndividualCacheDirectory(Context context) {
        return getIndividualCacheDirectory(context, INDIVIDUAL_DIR_NAME);
    }

    public static File getIndividualCacheDirectory(Context context, String cacheDir) {
        File appCacheDir = getCacheDirectory(context);
        File individualCacheDir = new File(appCacheDir, cacheDir);
        return (individualCacheDir.exists() || individualCacheDir.mkdir()) ? individualCacheDir : appCacheDir;
    }

    private static File getExternalCacheDir(Context context, String customDir) {
        File dataDir = new File(new File(Environment.getExternalStorageDirectory(), "Android"), "data");
        File appCacheDir = new File(new File(dataDir, context.getPackageName()), "cache");
        if (!appCacheDir.exists()) {
            if (!appCacheDir.mkdirs()) {
                // Unable to create external cache directory
                return null;
            }
            try {
                new File(appCacheDir, customDir).createNewFile();
            } catch (IOException e) {
            }
        }
        return appCacheDir;
    }

    public static boolean isAvailable() {
        return Environment.MEDIA_MOUNTED.equals(Environment.getExternalStorageState());
    }

    private static boolean hasExternalStoragePermission(Context context) {
        return context.checkCallingOrSelfPermission(EXTERNAL_STORAGE_PERMISSION) == PackageManager.PERMISSION_GRANTED;
    }
}
