package com.winhc.widget.util;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

/**
 * Create by Lee at 2021/9/25
 */
public class StringUtil {
    public final static String UTF_8 = "utf-8";
    static List<String> nullList = new ArrayList<>();

    static {
        nullList.add(null);
        nullList.add("");
        nullList.add("null");
        nullList.add("(null)");
        nullList.add("<null>");
    }

    public static boolean isEmpty(String value) {
        return isBlank(value) || nullList.contains(value);
    }

    public static boolean isBlank(String str) {
        return null == str || "".equals(str) || str.trim().length() == 0;
    }

    /**
     * @param obj
     * @return
     */
    public static boolean notNull(Object obj) {
        return null != obj && !nullList.contains(obj);
    }

    public static boolean isSpace(final String s) {
        if (s == null) {
            return true;
        }
        for (int i = 0, len = s.length(); i < len; ++i) {
            if (!Character.isWhitespace(s.charAt(i))) {
                return false;
            }
        }
        return true;
    }


    public static String nullStrToEmpty(Object str) {
        return (str instanceof String) ? (String) str : null == str ? "" : str.toString();
    }


    public static String capitalizeFirstLetter(String str) {
        if (isEmpty(str)) {
            return str;
        }
        char c = str.charAt(0);
        return (!Character.isLetter(c) || Character.isUpperCase(c)) ? str : Character.toUpperCase(c) + str.substring(1);
    }

    /**
     * 进行Encode编码
     *
     * @param target
     * @return
     */
    public static String urlEncode(String target) {
        return urlEncode(target, "UTF-8");
    }

    public static String urlEncode(String target, String charset) {
        try {
            return URLEncoder.encode(target, charset);
        } catch (UnsupportedEncodingException e) {
            return target;
        }
    }

    public static String urlDecode(String target) {
        return urlDecode(target, "UTF-8");
    }

    public static String urlDecode(String target, String charset) {
        try {
            return URLDecoder.decode(target, charset);
        } catch (UnsupportedEncodingException e) {
            return target;
        }
    }

    /**
     * 数据库替换
     *
     * @param keyWord
     * @return
     */
    public static String sqliteEscape(String keyWord) {
        keyWord = keyWord.replace("/", "//");
        keyWord = keyWord.replace("'", "''");
        keyWord = keyWord.replace("[", "/[");
        keyWord = keyWord.replace("]", "/]");
        keyWord = keyWord.replace("%", "/%");
        keyWord = keyWord.replace("&", "/&");
        keyWord = keyWord.replace("_", "/_");
        keyWord = keyWord.replace("(", "/(");
        keyWord = keyWord.replace(")", "/)");
        return keyWord;
    }
}
