package com.winhc.business.builder;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.TextView;

import com.winhc.business.R;
import com.winhc.widget.DialogBuilder;
import com.winhc.widget.util.StringUtil;

/**
 * Create by LZ at 2021/10/12
 */
public class CommonDialog {
    /**
     * 公共dialog的确定按钮的回调
     */
    public interface CommonDialogCallback {
        void onConfirmListener();

        void onCancelListener();
    }

    /**
     * @param context
     * @param titleText
     * @param messageText
     * @param leftText
     * @param rightText
     * @param callback
     */
    public static void showLoginDialog(Context context, String titleText, String messageText, String leftText, String rightText, final CommonDialogCallback callback) {
        View inflate = LayoutInflater.from(context).inflate(R.layout.dialog_common_layout, null, false);
        final DialogBuilder builder = new DialogBuilder(context)
                .build(inflate, R.style.dialogStyle)
                .cancelable(false)
                .outsideCanceled(true);
        TextView tvTitle = builder.findViewById(R.id.dialog_title);
        TextView tvMessage = builder.findViewById(R.id.dialog_message);
        TextView tvLeft = builder.findViewById(R.id.dial_cancel);
        View xian = builder.findViewById(R.id.xian);
        TextView tvRight = builder.findViewById(R.id.dial_confirm);

        if (!StringUtil.isEmpty(titleText)) {
            tvTitle.setText(titleText);
            tvTitle.setVisibility(View.VISIBLE);
        } else {
            tvTitle.setVisibility(View.GONE);
        }
        if (!StringUtil.isEmpty(messageText)) {
            tvMessage.setText(messageText);
            tvMessage.setVisibility(View.VISIBLE);
        } else {
            tvMessage.setVisibility(View.GONE);
        }
        if (!StringUtil.isEmpty(leftText)) {
            tvLeft.setText(leftText);
        }
        if (!StringUtil.isEmpty(rightText)) {
            tvRight.setText(rightText);
        }
        tvLeft.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                builder.cancel();
            }
        });
        tvRight.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (callback != null) {
                    callback.onConfirmListener();
                }
            }
        });
        builder.show();
    }

    /**
     * 更新的弹窗
     *
     * @param context
     * @param titleText
     * @param messageText
     * @param leftText
     * @param rightText
     * @param isCancelable
     * @param callback
     */
    public static void showUpdateDialog(Context context, String titleText, String messageText, String leftText, String rightText, boolean isCancelable, final CommonDialogCallback callback) {
        {
            View inflate = LayoutInflater.from(context).inflate(R.layout.dialog_common_layout, null, false);
            final DialogBuilder builder = new DialogBuilder(context)
                    .build(inflate, R.style.dialogStyle)
                    .cancelable(isCancelable);
            TextView tvTitle = builder.findViewById(R.id.dialog_title);
            TextView tvMessage = builder.findViewById(R.id.dialog_message);
            TextView tvLeft = builder.findViewById(R.id.dial_cancel);
            TextView tvRight = builder.findViewById(R.id.dial_confirm);

            if (!StringUtil.isEmpty(titleText)) {
                tvTitle.setText(titleText);
                tvTitle.setVisibility(View.VISIBLE);
            } else {
                tvTitle.setVisibility(View.GONE);
            }
            if (!StringUtil.isEmpty(messageText)) {
                tvMessage.setText(messageText);
                tvMessage.setVisibility(View.VISIBLE);
            } else {
                tvMessage.setVisibility(View.GONE);
            }
            if (!StringUtil.isEmpty(leftText)) {
                tvLeft.setText(leftText);
            }
            if (!StringUtil.isEmpty(rightText)) {
                tvRight.setText(rightText);
            }
            tvLeft.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    builder.cancel();
                    if (callback != null) {
                        callback.onCancelListener();
                    }
                }
            });
            tvRight.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    if (callback != null) {
                        callback.onConfirmListener();
                    }
                }
            });
            builder.show();
        }
    }
}
