package com.winhc.business.builder;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.InstanceCreator;
import com.google.gson.JsonElement;
import com.google.gson.JsonPrimitive;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;
import com.google.gson.TypeAdapter;
import com.google.gson.internal.ConstructorConstructor;
import com.google.gson.stream.JsonReader;
import com.google.gson.stream.JsonToken;
import com.google.gson.stream.JsonWriter;

import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.Type;
import java.util.Map;

public class HobbyGsonBuilder {

    private static HobbyGsonBuilder instance = null;
    private static GsonBuilder mGsonBuilder = null;

    private static String TAG = "EpartyGsonBuilder";

    private static HobbyGsonBuilder getInstance() {
        if (instance == null) {
            instance = new HobbyGsonBuilder();
        }

        return instance;
    }

    public synchronized static Gson getGson() {
        if (mGsonBuilder == null) {
            getInstance();
        }

        return mGsonBuilder.create();
    }

    private HobbyGsonBuilder() {
        init();
    }

    private void init() {
        mGsonBuilder = new GsonBuilder();
        mGsonBuilder.registerTypeAdapter(String.class, STRING);
        // mGsonBuilder.registerTypeAdapter(Integer.class, INTEGER);
        mGsonBuilder.registerTypeAdapter(int.class, INTEGER);
        // mGsonBuilder.registerTypeAdapter(Long.class, LONG);
        mGsonBuilder.registerTypeAdapter(long.class, LONG);
        // mGsonBuilder.registerTypeAdapter(Double.class, DOUBLE);
        mGsonBuilder.registerTypeAdapter(double.class, DOUBLE);
        mGsonBuilder.registerTypeAdapter(Double.class, new JsonSerializer<Double>() {

            @Override
            public JsonElement serialize(Double src, Type typeOfSrc, JsonSerializationContext context) {
                if (src == src.longValue())
                    return new JsonPrimitive(src.longValue());
                return new JsonPrimitive(src);
            }
        });

        //通过反射获取instanceCreators属性
        try {
            Class builder = mGsonBuilder.getClass();
            Field f = builder.getDeclaredField("instanceCreators");
            f.setAccessible(true);
            Map<Type, InstanceCreator<?>> val = (Map<Type, InstanceCreator<?>>) f.get(mGsonBuilder);//得到此属性的值
            //注册数组的处理器
            mGsonBuilder.registerTypeAdapterFactory(new CollectionTypeAdapterFactory(new ConstructorConstructor(val)));
        } catch (NoSuchFieldException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }

        // boolean 不能自定义，不知道 Gson nextBoolean 为什么有的时候不起作用，即使用了，也无法控制异常处理，暂时不要用了
        // mGsonBuilder.registerTypeAdapter(Boolean.class, BOOLEAN);
        // mGsonBuilder.registerTypeAdapter(boolean.class, BOOLEAN);
    }

  /*
   ** 备注：JsonReader 的原生解析函数会抛出 JsonSyntaxException 异常，如 nextDouble()，nextInt()等
   ** 如果想正常解析的同时又抛出 JsonSyntaxException 这个异常供外层使用，请使用原始的nextXXX()函数
   **/

    private static final TypeAdapter<String> STRING = new TypeAdapter<String>() {
        public String read(JsonReader reader) throws IOException {
            try {
                if (reader.peek() == JsonToken.NULL) {
                    reader.nextNull();
                    return "";// 原先是返回Null，这里改为返回空字符串
                }

                return reader.nextString();
            } catch (IllegalStateException e) {
                // e.printStackTrace();
                reader.skipValue();
                return "";
            } catch (Exception e) {
                // e.printStackTrace();
                return "";
            }
        }

        public void write(JsonWriter writer, String value) {
            try {
                if (value == null) {
                    writer.nullValue();
                    return;
                }
                writer.value(value);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    };

    private static final TypeAdapter<Integer> INTEGER = new TypeAdapter<Integer>() {
        public Integer read(JsonReader reader) throws IOException {
            try {
                // return reader.nextInt();
                if (reader.peek() == JsonToken.NULL) {
                    reader.nextNull();
                    return 0;
                }

                return Integer.parseInt(reader.nextString());
            } catch (IllegalStateException e) {
                // e.printStackTrace();
                reader.skipValue();
                return 0;
            } catch (Exception e) {
                // e.printStackTrace();
                return 0;
            }
        }

        public void write(JsonWriter writer, Integer value) {
            try {
                writer.value(String.valueOf(value));
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    };

    private static final TypeAdapter<Long> LONG = new TypeAdapter<Long>() {
        public Long read(JsonReader reader) throws IOException {
            try {
                // return reader.nextLong();
                if (reader.peek() == JsonToken.NULL) {
                    reader.nextNull();
                    return 0L;
                }

                return Long.valueOf(reader.nextString());
            } catch (IllegalStateException e) {
                // e.printStackTrace();
                reader.skipValue();
                return 0L;
            } catch (Exception e) {
                // e.printStackTrace();
                return 0L;
            }
        }

        public void write(JsonWriter writer, Long value) {
            try {
                writer.value(String.valueOf(value));
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    };

    private static final TypeAdapter<Double> DOUBLE = new TypeAdapter<Double>() {
        public Double read(JsonReader reader) throws IOException {
            try {
                // value = reader.nextDouble();
                if (reader.peek() == JsonToken.NULL) {
                    reader.nextNull();
                    return 0.0D;
                }

                return Double.valueOf(reader.nextString());
            } catch (IllegalStateException e) {
                // e.printStackTrace();
                reader.skipValue();
                return 0.0D;
            } catch (Exception e) {
                // e.printStackTrace();
                return 0.0D;
            }
        }

        public void write(JsonWriter writer, Double value) {
            try {
                writer.value(String.valueOf(value));
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    };

}
