package com.winhc.business.utils;

import android.Manifest;
import android.content.Context;
import android.content.pm.PackageManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Build;
import android.os.Environment;
import android.telephony.TelephonyManager;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.UUID;

/**
 * 与设备相关的工具类
 */

public class DeviceUtil {

    /**
     * Check network
     *
     * @param context
     * @return
     */
    public static boolean checkNetwork(Context context) {
        ConnectivityManager connectivityManager = (ConnectivityManager) context
                .getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo networkInfo = connectivityManager == null ? null : connectivityManager.getActiveNetworkInfo();
        return networkInfo != null && networkInfo.isConnected();
    }

    /**
     * 获得设备的IMEI
     */
    public static String getDeviceId(Context context) {
        String imei = getDeviceIMEI(context);
        String imsi = getIMSI(context);
        String salt = "123456";
        String deviceID = MD5Utils.string2MD5(imei + imsi + salt);

        return deviceID;
    }

    public static String getUniquePsuedoID() {
        String serial = null;

        String m_szDevIDShort = "35" +
                Build.BOARD.length() % 10 + Build.BRAND.length() % 10 +

                Build.CPU_ABI.length() % 10 + Build.DEVICE.length() % 10 +

                Build.DISPLAY.length() % 10 + Build.HOST.length() % 10 +

                Build.ID.length() % 10 + Build.MANUFACTURER.length() % 10 +

                Build.MODEL.length() % 10 + Build.PRODUCT.length() % 10 +

                Build.TAGS.length() % 10 + Build.TYPE.length() % 10 +

                Build.USER.length() % 10; //13 位

        try {
            serial = Build.class.getField("SERIAL").get(null).toString();
            //API>=9 使用serial号
            return new UUID(m_szDevIDShort.hashCode(), serial.hashCode()).toString();
        } catch (Exception exception) {
            //serial需要一个初始化
            serial = "serial"; // 随便一个初始化
        }
        //使用硬件信息拼凑出来的15位号码
        return new UUID(m_szDevIDShort.hashCode(), serial.hashCode()).toString();
    }

    public static String getOsVersion() {
        String result = Build.VERSION.RELEASE;
        if (result == null){
            return "";
        }

        return result;
    }

    /**
     * 获取手机型号
     *
     * @return 手机型号
     */
    public static String getSystemModel() {
        return Build.MODEL;
    }

    private static String getDeviceIMEI(Context context) {
        String result = "";
        try {
            if (!checkPermissions(context,
                    Manifest.permission.READ_PHONE_STATE)) {

                return "";
            }
            TelephonyManager telephonyManager = (TelephonyManager) (context
                    .getSystemService(Context.TELEPHONY_SERVICE));
            result = telephonyManager.getDeviceId();
            if (result == null)
                result = "";
        } catch (Exception e) {

        }
        return result;
    }


    /**
     * get IMSI for GSM phone, return "" if it is unavailable.
     *
     * @return IMSI string
     */
    public static String getIMSI(Context context) {
        String result = "";
        try {
            if (!checkPermissions(context,
                    Manifest.permission.READ_PHONE_STATE)) {

                return "";
            }
            TelephonyManager telephonyManager = (TelephonyManager) (context
                    .getSystemService(Context.TELEPHONY_SERVICE));
            result = telephonyManager.getSubscriberId();

            if (result == null)
                return "";
            return result;

        } catch (Exception e) {
        }

        return result;
    }

    /**
     * 返回该设备在此程序上的随机数。
     *
     * @param context Context对象。
     * @return 表示该设备在此程序上的随机数。
     */
    public synchronized static String getSALT(Context context) {
        String file_name = context.getPackageName().replace(".", "");
        String sdCardRoot = Environment.getExternalStorageDirectory()
                .getAbsolutePath();
        int apiLevel = Integer.parseInt(Build.VERSION.SDK);
        File fileFromSDCard = new File(sdCardRoot + File.separator, "." + file_name);
        File fileFromDData = new File(context.getFilesDir(), file_name);// 获取data/data/<package>/files
        //4.4之後 /storage/emulated/0/Android/data/<package>/files
        if (apiLevel >= 19) {
            sdCardRoot = context.getExternalFilesDir(null).getAbsolutePath();
            fileFromSDCard = new File(sdCardRoot, file_name);
        }

        String saltString = "";
        if (Environment.getExternalStorageState().equals(
                Environment.MEDIA_MOUNTED)) {
            // sdcard存在
            if (!fileFromSDCard.exists()) {

                String saltId = getSaltOnDataData(fileFromDData, file_name);
                try {
                    writeToFile(fileFromSDCard, saltId);
                } catch (Exception e) {
                }
                return saltId;

            } else {
                // SD卡上存在salt
                saltString = getSaltOnSDCard(fileFromSDCard);
                try {
                    writeToFile(fileFromDData, saltString);
                } catch (IOException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
                return saltString;
            }

        } else {
            // sdcard 不可用
            return getSaltOnDataData(fileFromDData, file_name);
        }

    }

    private static String getSaltOnSDCard(File fileFromSDCard) {
        // TODO Auto-generated method stub
        try {
            return readSaltFromFile(fileFromSDCard);
        } catch (IOException e) {
        }
        return null;
    }

    private static String getSaltOnDataData(File fileFromDData, String file_name) {
        try {
            if (!fileFromDData.exists()) {
                String uuid = getUUID();
                writeToFile(fileFromDData, uuid);
                return uuid;
            }
            return readSaltFromFile(fileFromDData);

        } catch (IOException e) {
        }
        return "";
    }

    private static String getUUID() {
        // TODO Auto-generated method stub
        return UUID.randomUUID().toString().replace("-", "");
    }

    /**
     * 读出保存在程序文件系统中的表示该设备在此程序上的唯一标识符。。
     *
     * @param file 保存唯一标识符的File对象。
     * @return 唯一标识符。
     * @throws IOException IO异常。
     */
    private static String readSaltFromFile(File file) throws IOException {
        RandomAccessFile accessFile = new RandomAccessFile(file, "r");
        byte[] bs = new byte[(int) accessFile.length()];
        accessFile.readFully(bs);
        accessFile.close();
        return new String(bs);
    }

    /**
     * 将表示此设备在该程序上的唯一标识符写入程序文件系统中
     *
     * @param uuid uuid
     * @param file 保存唯一标识符的File对象。
     * @throws IOException IO异常。
     */
    private static void writeToFile(File file, String uuid)
            throws IOException {
        file.createNewFile();
        FileOutputStream out = new FileOutputStream(file);

        out.write(uuid.getBytes());
        out.close();

    }

    /**
     * checkPermissions
     *
     * @param context
     * @param permission
     * @return true or false
     */
    public static boolean checkPermissions(Context context, String permission) {
        if (context == null || permission.equals("") || permission.equals("")) {
            return false;
        }
        PackageManager pm = context.getPackageManager();
        return pm.checkPermission(permission, context.getPackageName())
                == PackageManager.PERMISSION_GRANTED;
    }

    /**
     * 生成唯一号
     *
     * @return
     */
    public static String get36UUID() {
        UUID uuid = UUID.randomUUID();
        String uniqueId = uuid.toString();
        return uniqueId;
    }

}
