package com.winhc.business.utils;

import android.content.Context;
import android.content.res.AssetManager;
import android.text.TextUtils;

import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.winhc.business.builder.HobbyGsonBuilder;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

/**
 * 全局使用的 Gson ，避免重复创建。
 */
public class GsonUtils {
    private static Gson sGson = HobbyGsonBuilder.getGson();

    public static Gson getGson() {
        return sGson;
    }

    public static String getJson(Context context, String fileName) {

        StringBuilder stringBuilder = new StringBuilder();
        try {
            AssetManager assetManager = context.getAssets();
            BufferedReader bf = new BufferedReader(new InputStreamReader(
                    assetManager.open(fileName)));
            String line;
            while ((line = bf.readLine()) != null) {
                stringBuilder.append(line);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return stringBuilder.toString();
    }

    //json解析某个字段成为bean而非 LinkedTreeMap
    public static <T> List<T> jsonToDto(String message, String jsonHead, Class<T> cls){ //这里是Class<T>
        try {
            JsonObject jsonObject = new JsonParser().parse(message).getAsJsonObject();
            JsonArray jsonArray = jsonObject.getAsJsonArray(jsonHead);

            List<T> list = new ArrayList<>();
            for (JsonElement jsonElement : jsonArray) {
                list.add(sGson.fromJson(jsonElement, cls)); //cls
            }
            return list;
        }catch (Exception e) {
            return null;
        }
    }

    /**
     * 单独获取json内的某个属性值
     * @param jsonStr
     * @param key
     * @return
     */
    public static String getKeyValue(String jsonStr,String key){
        if (TextUtils.isEmpty(jsonStr)) return "--";
        try {
            JsonElement  je = new JsonParser().parse(jsonStr);
            return TextUtils.isEmpty(je.getAsJsonObject().get(key).getAsString())?"--":je.getAsJsonObject().get(key).getAsString();
        }catch (Exception e){
            return "--";
        }
    }

    public static String getKeyValue2(String jsonStr,String key){
        if (TextUtils.isEmpty(jsonStr)) return "";
        try {
            JsonElement  je = new JsonParser().parse(jsonStr);
            return TextUtils.isEmpty(je.getAsJsonObject().get(key).getAsString())?"":je.getAsJsonObject().get(key).getAsString();
        }catch (Exception e){
            return "";
        }
    }

}
