package com.winhc.business.utils;

import java.math.BigDecimal;
import java.text.DecimalFormat;

/**
 * 用于格式化金额
 */
public class MoneyUtil {
    public static boolean isEmpty(String value) {
        return null == value || "null".equals(value) || "".equals(value) || value.length() == 0;
    }

    /**
     * 金额转换
     *
     * @param value
     * @return
     */
    public static long parseLongValue(String value) {
        if (isEmpty(value)) {
            return 0;
        }
        long result = 0;
        try {
            result = Long.parseLong(value);
        } catch (Exception ignored) {

        }
        return result;
    }

    /**
     * 金额转换
     *
     * @param value
     * @return
     */
    public static float parseFloatValue(String value) {
        if (isEmpty(value)) {
            return 0.0f;
        }
        float result = 0.0f;
        try {
            result = Float.parseFloat(value);
        } catch (Exception ignored) {

        }
        return result;
    }

    /**
     * 金额转换
     *
     * @param value
     * @return
     */
    public static double parseValue(String value) {
        if (isEmpty(value)) {
            return 0.0;
        }
        double result = 0.0;
        try {
            result = Double.parseDouble(value);
        } catch (Exception ignored) {

        }
        return result;
    }

    /**
     * @param money
     * @return
     */
    public static String formatMoney(String money) {
        double result = parseValue(money);
        if (result <= 0.0) {
            return "0.00";
        }
        DecimalFormat decimalFormat = new DecimalFormat("###,###.00");
        return decimalFormat.format(result);
    }

    /**
     * @param money
     * @return
     */
    public static String formatMoney(double money) {
        if (money <= 0.0) {
            return "0.00";
        }
        DecimalFormat decimalFormat = new DecimalFormat("###,###.00");
        return decimalFormat.format(money);
    }
    private static final int DEF_DIV_SCALE = 10;

    /**
     * * 两个Double数相加 *
     *
     * @param v1 *
     * @param v2 *
     * @return Double
     */
    public static Double add(Double v1, Double v2) {
        BigDecimal b1 = new BigDecimal(v1.toString());
        BigDecimal b2 = new BigDecimal(v2.toString());
        return new Double(b1.add(b2).doubleValue());
    }

    /**
     * * 两个Double数相减 *
     *
     * @param v1 *
     * @param v2 *
     * @return Double
     */
    public static Double sub(Double v1, Double v2) {
        BigDecimal b1 = new BigDecimal(v1.toString());
        BigDecimal b2 = new BigDecimal(v2.toString());
        return new Double(b1.subtract(b2).doubleValue());
    }

    /**
     * * 两个Double数相乘 *
     *
     * @param v1 *
     * @param v2 *
     * @return Double
     */
    public static Double mul(Double v1, Double v2) {
        BigDecimal b1 = new BigDecimal(v1.toString());
        BigDecimal b2 = new BigDecimal(v2.toString());
        return new Double(b1.multiply(b2).doubleValue());
    }

    /**
     * * 两个Double数相除 *
     *
     * @param v1 *
     * @param v2 *
     * @return Double
     */
    public static Double div(Double v1, Double v2) {
        BigDecimal b1 = new BigDecimal(v1.toString());
        BigDecimal b2 = new BigDecimal(v2.toString());
        return new Double(b1.divide(b2, DEF_DIV_SCALE, BigDecimal.ROUND_HALF_UP)
                .doubleValue());
    }

    /**
     * * 两个Double数相除，并保留scale位小数 *
     *
     * @param v1    *
     * @param v2    *
     * @param scale *
     * @return Double
     */
    public static Double div(Double v1, Double v2, int scale) {
        if (scale < 0) {
            throw new IllegalArgumentException(
                    "The scale must be a positive integer or zero");
        }
        BigDecimal b1 = new BigDecimal(v1.toString());
        BigDecimal b2 = new BigDecimal(v2.toString());
        return new Double(b1.divide(b2, scale, BigDecimal.ROUND_HALF_UP).doubleValue());
    }
}
