package com.winhc.business.utils;

import android.text.TextUtils;

import com.sensorsdata.analytics.android.sdk.SensorsDataAPI;
import com.winhc.widget.util.MMLog;
import com.winhc.widget.util.StringUtil;

import org.json.JSONArray;
import org.json.JSONObject;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Created by toby on 2020/9/7.
 */
public class SensorDataEventUtils {

    /**
     * 注册成功
     */
    public static void addRegistSuccessEvent(String type) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("regist_method", type);
//            properties.put("is_success", isSuccess);
//            properties.put("fail_reason", errorMsg);
//            properties.put("DownloadChannel", ValidatesUtil.isEmpty(channel) ? "guanwang" : channel);
//            properties.put("is_new", AccountManager.getInstance().getAccount().newUser);
            track("register_result", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 是否登陆成功
     *
     * @param type
     * @param fail_reason
     * @param isSuccess
     */
    public static void addLoginResultEvent(String type, String fail_reason, boolean isSuccess) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("loading_method", type);
            properties.put("fail_reason", fail_reason);
            properties.put("is_success", isSuccess);
            track("is_register_success", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 浏览报告生成页面埋点
     *
     * @param report_name_list
     * @param source_page_name
     */
    public static void addReportBrowseEvent(String report_name_list, String source_page_name) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("report_name_list", report_name_list);
            properties.put("source_page_name", source_page_name);
            track("report_export_page_view", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 点击生成报告埋点
     *
     * @param reportType
     * @param isNeedPay
     */
    public static void addReportEvent(String reportType, boolean isNeedPay) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("report_type", reportType);
            properties.put("is_payment_needed", isNeedPay);
            track("report_pay_page_view", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 提交订单埋点
     */
    public static void addCommitOrderEvent(String product_name, String payment_method, double payment_amount) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("product_name", product_name);
            properties.put("payment_method", payment_method);
            properties.put("payment_amount", payment_amount);
            track("confirm_order_submission", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 确认支付订单埋点
     */
    public static void addConfirmOrderEvent(String order_type, String product_name, String payment_method, double payment_amount, String order_ID) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("order_type", order_type);
            properties.put("product_name", product_name);
            properties.put("payment_method", payment_method);
            properties.put("payment_amount", payment_amount);
            properties.put("order_ID", order_ID);
            track("confirm_payment_order", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * Banner点击埋点
     */
    public static void addBannerClickEvent(String page_name, String campaign_name, String campaign_link, int campaign_rank, String campaign_id) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("page_name", page_name);
            properties.put("campaign_name", campaign_name);
            properties.put("campaign_link", campaign_link);
            properties.put("campaign_rank", campaign_rank);
            properties.put("campaign_id", campaign_id);
            track("banner_click", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 首页广告、启动页广告点击埋点
     */
    public static void addAdvClickEvent(String event_name, String campaign_name, String campaign_link, String campaign_id) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("campaign_name", campaign_name);
            properties.put("campaign_link", campaign_link);
            properties.put("campaign_id", campaign_id);
            track(event_name, properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * push通知点击埋点
     */
    public static void addPushClickEvent(String redirect_page_name, String push_title) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("redirect_page_name", redirect_page_name);
            properties.put("push_title", push_title);
            track("push_click", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 认证点击埋点
     */
    public static void addVerifyClickEvent(String eventName, String attrName, String attrValue) {
        try {
            JSONObject properties = new JSONObject();
            properties.put(attrName, attrValue);
            track(eventName, properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 分享点击埋点
     */
    public static void addShareClickEvent(String page_name, String share_method, String share_content_type, String share_content_title) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("page_name", page_name);
            properties.put("share_method", share_method);
            properties.put("share_content_type", share_content_type);
            properties.put("share_content_title", share_content_title);
            track("share_click", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 工作台发布点击埋点
     */
    public static void addPublishWorkClickEvent(String publish_type, String reminder_type) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("publish_type", publish_type);
            properties.put("reminder_type", reminder_type);
            track("workbench_release_page", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 工作台发布提醒成功点击埋点
     */
    public static void addPublishTiXingSuccessClickEvent(String reminder_type) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("reminder_type", reminder_type);
            track("reminder_set", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 工作台发布随手记成功点击埋点
     */
    public static void addPublishNoteSuccessClickEvent(int tag_number, boolean is_company_related, boolean is_pic_uploaded, boolean is_time_selected,
                                                       boolean is_address_seleted, boolean is_report_turned_on) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("is_company_related", is_company_related);
            properties.put("is_pic_uploaded", is_pic_uploaded);
            properties.put("is_time_selected", is_time_selected);
            properties.put("is_address_seleted", is_address_seleted);
            properties.put("is_report_turned_on", is_report_turned_on);
            properties.put("tag_number", tag_number);
            track("note_taking_add", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 尽职调查报告点击埋点
     */
    public static void addJINZHIDIAOCHAClickEvent(String sample_type_view, String report_type_produce) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("sample_type_view", sample_type_view);
            properties.put("report_type_produce", report_type_produce);
            track("due_diligence_report_view", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 异地尽调点击埋点
     */
    public static void addEntrustClickEvent(String eventName, String DD_area, String DD_type) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("DD_area", DD_area);
            properties.put("DD_type", DD_type);
            track(eventName, properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 旧案托管点击埋点
     */
    public static void addTuoguanClickEvent(String eventName, String custody_type) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("custody_type", custody_type);
            track(eventName, properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 消息模块点击
     *
     * @param pageName
     */
    public static void addMessageEvent(String pageName) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("source_page_name", pageName);
            track("msg_module_click", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 消息列表点击
     *
     * @param msg_list_name
     */
    public static void addMessageListEvent(String msg_list_name) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("msg_list_name", msg_list_name);
            track("msg_list", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 全部功能管理
     */
    public static void addAllfuctionStart(List<String> functionList) {
        try {
            JSONObject properties = new JSONObject();
            JSONArray array = new JSONArray();
            for (String value : functionList) {
                array.put(value);
            }
            properties.put("my_function_selected", array);
            track("full_function_manage", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 全部功能完成
     */
    public static void addAllfuction(List<String> functionList) {
        try {
            JSONObject properties = new JSONObject();
            JSONArray array = new JSONArray();
            for (String value : functionList) {
                array.put(value);
            }
            properties.put("my_function_selected", array);
            track("complete_function_manage", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 案源库筛选项选择
     */
    public static void addCaseSourceSelect(String areaSelected, String amountSelected) {
        try {
            JSONObject properties = new JSONObject();
            if (TextUtils.isEmpty(areaSelected)) {
                areaSelected = "全国";
            }

            if (TextUtils.isEmpty(amountSelected)) {
                amountSelected = "标的不限";
            }
            properties.put("area_filter_selected", areaSelected);
            properties.put("target_amount_selected", amountSelected);
            track("case_database_filter_set", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 案源库详情浏览
     */
    public static void addCaseSourceDetail(List<String> arreaList, String amountSelected) {
        try {
            JSONObject properties = new JSONObject();
            JSONArray array = new JSONArray();
            for (String value : arreaList) {
                array.put(value);
            }
            properties.put("case_area", array);
            properties.put("case_target_amount", amountSelected);
            track("case_detail_page_view", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 案源库申请成功
     */
    public static void addCaseSourceApply(List<String> arreaList, String amountSelected) {
        try {
            JSONObject properties = new JSONObject();
            JSONArray array = new JSONArray();
            for (String value : arreaList) {
                array.put(value);
            }
            properties.put("case_area", array);
            properties.put("case_target_amount", amountSelected);
            track("case_database_coop_apply_success", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 案源库分享
     */
    public static void addCaseSourceShare(String status, String type) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("request_status", status);
            properties.put("share_type", type);
            track("case_database_share", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 添加监测企业
     */
    public static void addCompanyMonitor(String pageName, String companyName) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("current_page_name", pageName);
            properties.put("company_name", companyName);
            track("monitor_company_add", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 类案检索搜索结果页
     */
    public static void addCaseRetrievalResult(String type, List<String> keyword, String reason, String court) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("keyword_type", type);
            JSONArray array = new JSONArray();
            for (String value : keyword) {
                array.put(value);
            }
            properties.put("case_keyword", array);
            properties.put("case_reason", reason);
            properties.put("administration_court", court);
            track("case_retrieval_result", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 搜索结果列表页浏览
     */
    public static void addSearchList(String type, boolean isHistory, String keyword) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("search_type", type);
            properties.put("is_history_search", isHistory);
            properties.put("keyword", keyword);
            track("search_result_list_click", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 搜索结果详情页浏览
     * 是否历史浏览
     */
    public static void addSearchDetail(String type, boolean isHistory, String keyword) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("search_type", type);
            properties.put("is_history_view", isHistory);
            properties.put("keyword", keyword);
            track("search_result_detail_pv", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 我要律师、评估律师
     */
    public static void addLawyerMatch(String key, String court, String reason, String target) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("administration_court", court);
            properties.put("case_reason", reason);
            properties.put("target_interval", target);
            track(key, properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 律师详情
     */
    public static void addLawyerMatchDetail(String lawyerName, String court) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("lawyer_name", lawyerName);
            properties.put("lawyer_court", court);
            track("browse_lawyer_details", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 律师详情 点击认领
     */
    public static void addLawyerMatchRenLing(String lawyerName, String court, String is_match) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("lawyer_name", lawyerName);
            properties.put("lawyer_court", court);
            properties.put("is_match", is_match);
            track("click_claim_button", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 律师详情 编辑
     */
    public static void addLawyerEdit(String type) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("lawyer_details_edit", type);
            track("lawyer_details_save", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 关注埋点
     */
    public static void addFocus(String follow_object) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("follow_object", follow_object);
            track("click_follow_button", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 首页顶部按钮点击
     */
    public static void addTopButton(String btnName) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("top_button_name", btnName);
            track("click_top_button", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 大数据查询模块按钮点击
     */
    public static void addBigDataButton(String btnName) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("big_data_button_name", btnName);
            track("click_big_data", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 计算器模块按钮点击
     */
    public static void addCalculatorButton(String btnName) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("calculator_button_name", btnName);
            track("click_calculator", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 常用功能模块按钮点击
     */
    public static void addCommonFunctionButton(String btnName) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("all_button_name", btnName);
            track("click_common_function", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 探月实验室模块按钮点击
     */
    public static void addMoonLaboratoryButton(String btnName) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("moon_laboratory_button_name", btnName);
            track("click_moon_laboratory", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 全部功能模块按钮点击
     */
    public static void addAllFunctionButton(String btnName) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("all_button_name", btnName);
            track("click_all_function", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 我的页面点击
     */
    public static void minPage(String clickName, String eventName, String buttonName) {
        try {
            JSONObject properties = new JSONObject();
            properties.put(buttonName, eventName);
            track(clickName, properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     *
     */
    public static void cooperationPage(String clickName, String collaboration_type, Double quotation_amount) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("collaboration_type", collaboration_type);
            properties.put("quotation_amount", quotation_amount);
            track(clickName, properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    public static void cooperationPayPage(String clickName, String collaboration_type, Double payment_amount) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("collaboration_type", collaboration_type);
            properties.put("payment_amount", payment_amount);
            track(clickName, properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    public static void addRecommandLawBtn(String module_type, String lawyer_name, String lawyer_area) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("module_type", module_type);
            properties.put("lawyer_name", lawyer_name);
            properties.put("lawyer_area", lawyer_area);
            track("preferred_awyer_click", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 红包相关
     */
    public static void redPackagePop(String eventName, String trigger_position) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("trigger_position", trigger_position);
            track(eventName, properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    public static void redPackageClose(String eventName, String trigger_position, String red_packet_sort, String red_packet_type) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("trigger_position", trigger_position);
            properties.put("red_packet_sort", red_packet_sort);
            properties.put("red_packet_type", red_packet_type);
            track(eventName, properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 红包开启,新春红包
     */
    public static void redPackageOpen(String eventName, String trigger_position, String red_packet_sort) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("trigger_position", trigger_position);
            properties.put("red_packet_sort", red_packet_sort);
            track(eventName, properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 红包开启,新春红包，红包打开页面按钮点击
     */
    public static void redPackageOpenButton(String eventName, String trigger_position, String red_packet_type, String button_name) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("trigger_position", trigger_position);
            properties.put("red_packet_type", red_packet_type);
            properties.put("button_name", button_name);
            track(eventName, properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 红包相关
     */
    public static void redPackageUse(String eventName, String trigger_position, String red_packet_type) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("trigger_position", trigger_position);
            properties.put("red_packet_type", red_packet_type);
            track(eventName, properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 浏览vip购买页
     *
     * @param prePage
     */
    public static void addVipPrePay(String prePage) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("source_page_name", prePage);
            track("vip_pay_page_view", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 司法拍卖-搜索结果列表
     *
     * @param method
     */
    public static void addJudicialDetail(String method) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("search_method", method);
            track("Judicial_search_list_click", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 司法拍卖-拍卖详情点击
     *
     * @param keyword
     */
    public static void addJudicialDetail_condition(List<String> keyword) {
        try {
            JSONObject properties = new JSONObject();
            JSONArray array = new JSONArray();
            for (String value : keyword) {
                array.put(value);
            }
            properties.put("filter_options", array);
            track("Judicial_details_click", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 司法拍卖-添加智能推送点击
     *
     * @param method
     */
    public static void addJudicialNoticePush(String method) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("trigger_button_position", method);
            track("Judicial_push_click", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 司法拍卖-添加智能推送成功
     *
     * @param keyword
     */
    public static void addJudicialNoticeContent(List<String> keyword) {
        try {
            JSONObject properties = new JSONObject();
            JSONArray array = new JSONArray();
            for (String value : keyword) {
                array.put(value);
            }
            properties.put("push_filling_content", array);
            track("Judicial_push_success", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 司法拍卖-智能推送点击
     *
     * @param key
     * @param location
     */
    public static void addJudicialNoticePushClick(String key, String location) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("push_keywords", key);
            properties.put("push_location", location);
            track("Judicial_smart_push_click", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 智慧诉讼-填写信息页面
     *
     * @param key
     */
    public static void addAnjiansanbao(String key) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("nature_case", key);
            track("smart_information_page", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 智慧诉讼-方案详情
     *
     * @param key
     * @param key2
     */
    public static void addAnjiansanbao2(String key, String key2) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("nature_case", key);
            properties.put("details_entry", key2);
            track("smart_case_details", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 智慧诉讼-生成方案报告点击
     *
     * @param key
     */
    public static void addAnjiansanbao3(String key) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("nature_case", key);
            track("smart_report_click", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 智慧执行-方案详情
     *
     * @param key
     */
    public static void addAnjiansanbao4(String key) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("nature_case", key);
            track("intelligent_execution_details", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 工作台顶部按钮点击
     *
     * @param key
     */
    public static void addAnjiansanbao5(String key) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("button_name", key);
            track("workbench_top_button_click", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 内侧中按钮点击
     *
     * @param key
     */
    public static void addAnjiansanbao6(String key) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("function_name", key);
            track("test_button_click", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    //司法案件列表,司法案件详情,查失信-查失信人列表,查失信-查被执行人列表,查被执-被执行人列表,查被执-查被执企业列表
    public static void searchList(String name, String eventName, List<String> functionList) {
        try {
            JSONObject properties = new JSONObject();
            JSONArray array = new JSONArray();
            for (String value : functionList) {
                array.put(value);
            }
            properties.put(name, array);
            track(eventName, properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    //失信,被执行详情
    public static void deadBeatDeatils(String details_type, String details_entry, String eventName, List<String> list) {
        try {
            JSONObject properties = new JSONObject();
            JSONArray array = new JSONArray();
            for (String value : list) {
                array.put(value);
            }
            properties.put("details_type", details_type);
            properties.put("details_entry", details_entry);
            properties.put("filter_options", array);
            track(eventName, properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    //失信信息,被执行信息详情,等一般埋点事件
    public static void simpleClick(String eventName) {
        try {
            track(eventName, null);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    //智慧法顾-生成报告点击
    public static void legal_service_report_click(boolean fill_custom_item) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("fill_custom_item", fill_custom_item);
            track("legal_service_report_click", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 企业法眼详情
     *
     * @param details_entry
     */
    public static void addLawEyes1(String details_entry) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("details_entry", details_entry);
            track("corporate_law_details", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 企业法眼-查看裁判文书
     *
     * @param page_entry
     */
    public static void addLawEyes2(String page_entry) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("page_entry", page_entry);
            track("corporate_law_judgment_document", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 企业法眼按钮点击
     *
     * @param button_name
     */
    public static void addLawEyes3(String button_name) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("button_name", button_name);
            track("corporate_law_button_click", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 合同模版搜索结果页
     *
     * @param source_modular
     */
    public static void contractList(String source_modular) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("source_modular", source_modular);
            track("contract_modular_search_result", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 合同模版详情
     *
     * @param contract_modular_name
     */
    public static void contractDetail(String contract_modular_name) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("contract_modular_name", contract_modular_name);
            track("contract_modular_details_click", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 账款精灵_介绍页点击
     *
     * @param button_name 限时体验、开始账款管理
     */
    public static void wizardSensor1(String button_name) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("button_name", button_name);
            track("account_introduction_click", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 账款精灵_添加账款成功
     *
     * @param add_type       智能机器人添加、列表添加
     * @param account_status 未逾期、已逾期
     * @param amount_account 账款金额
     * @param judicial_stage 未诉讼、已诉讼、已申请执行、终结本次执行
     * @param account_date   应收账款日期
     */
    public static void wizardSensor2(String add_type, String account_status, String amount_account, String judicial_stage, String account_date) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("add_type", add_type);
            properties.put("account_status", account_status);
            properties.put("amount_account", amount_account);
            properties.put("judicial_stage", judicial_stage);
            properties.put("account_date", account_date);
            track("account_success_add", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 账款精灵_智能提醒详情
     *
     * @param source_page_name 账款精灵列表页、账款精灵详情页、智能提醒列表页、消息列表
     * @param reminder_type    逾期提醒、财产线索、评级变化、风险提醒
     */
    public static void wizardSensor3(String source_page_name, String reminder_type) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("source_page_name", source_page_name);
            properties.put("reminder_type", reminder_type);
            track("account_reminder_details", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 账款精灵_增值服务点击
     *
     * @param source_page_name 账款详情、智能提醒详情
     * @param button_name      案件诊断、诉讼投资、搜律师、尽职调查、找财产、查风险等
     */
    public static void wizardSensor4(String source_page_name, String button_name) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("source_page_name", source_page_name);
            properties.put("button_name", button_name);
            track("account_value_service_click", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 账款精灵_添加回收金额
     */
    public static void wizardSensor5() {
        track("account_add_aecovery", null);
    }

    /**
     * 账款精灵_手势密码设置
     *
     * @param setting_method 初次设置、列表页设置
     */
    public static void wizardSensor6(String setting_method) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("setting_method", setting_method);
            track("account_password_setting", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    public static void wizardSensor7(String source_page_name) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("source_page_name", source_page_name);
            track("account_reminder_list", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    public static void wizardSensor8(String function_name) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("function_name", function_name);
            track("account_Smart_drainage", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    public static void wizardSensor9(String source_page_name) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("source_page_name", source_page_name);
            track("account_function_description_browse", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 首页顶部律师信息点击
     *
     * @param button_name
     */
    public static void addLawHomeBtn(String button_name) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("button_name", button_name);
            track("lawyer_info_homepage_click", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 案源广场相关埋点
     *
     * @param eventName
     * @param service_type
     * @param case_type
     */
    public static void addAnYuanBtn(String eventName, String service_type, String case_type) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("service_type", service_type);
            properties.put("case_type", case_type);
            track(eventName, properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    public static void addAnYuanBtn(String eventName, String service_type, String case_type, String service_class,
                                    String publish_time, String case_money) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("service_type", service_type);
            properties.put("case_type", case_type);
            properties.put("service_class", service_class);
            properties.put("publish_time", publish_time);
            properties.put("case_money", case_money);
            track(eventName, properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 单纯点击事件
     *
     * @param eventName
     */
    public static void addCommonClick(String eventName) {
        track(eventName, null);
    }

    /**
     * 一问多答，行业专家请教
     *
     * @param eventName industry_type
     *                  question_type
     *                  reward_amount
     */
    public static void addAskSuccess(String eventName, String industry_type, String question_type, String reward_amount) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("industry_type", industry_type);
            properties.put("question_type", question_type);
            properties.put("reward_amount", reward_amount);
            track(eventName, properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 法律咨询点击立即下单
     * consult_type
     */
    public static void addServiceOrderClick(String product_name) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("product_name", product_name);
            track("legal_service_click_order", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 法律咨询点击下单
     * consult_type
     */
    public static void addconsultOrderClick(String question_type, String product_name) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("question_type", question_type);
            properties.put("product_name", product_name);
            track("legal_consultation_click_order", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 法律服务，合作
     */
    public static void addLawCooperaClick(String eventName, String service_type, String product_name) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("service_type", service_type);
            properties.put("product_name", product_name);
            track(eventName, properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 点击提交评价
     */
    public static void addCommentClick(int review_star, String order_ID, String product_name) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("review_star", review_star);
            properties.put("order_ID", order_ID);
            properties.put("product_name", product_name);
            track("click_submit_review", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 云信错误统计提交
     */
    public static void reportYunXinError(String error_message, String business_name) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("error_message", error_message);
            properties.put("business_name", business_name);
            track("report_errors_event", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 到法律咨询入口
     */
    public static void aiZixunPageClick(String button_name) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("button_name", button_name);
            track("AI_entrance_click", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 催TA上线
     *
     * @param value
     */
    public static void addCuitasx(boolean value) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("is_contact", value);
            track("urging_TA_online", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 快速咨询页面浏览
     *
     * @param value
     * @param value1
     */
    public static void addBrowseConsult(String value, String value1) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("source_page_name", value);
            properties.put("default_option", value1);
            track("quick_page_browsing", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 一问多答下单页面浏览
     *
     * @param value
     */
    public static void addBrowseOneAsk(String value) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("source_page_name", value);
            track("one_page_brosing", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 查失信限高-详情浏览
     */
    public static void checkDisHonestDetail(String details_type, List<String> filter_options) {
        try {
            JSONObject properties = new JSONObject();
            JSONArray array = new JSONArray();
            for (String value : filter_options) {
                array.put(value);
            }
            properties.put("details_type", details_type);
            properties.put("filter_options", array);
            track("check_dishonest_details", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 首页订单入口点击
     *
     * @param order_type
     * @param prompt_copy
     * @param order_num
     */
    public static void indexOrderEvent(String order_type, String prompt_copy, int order_num) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("order_type", order_type);
            properties.put("prompt_copy", prompt_copy);
            properties.put("order_num", order_num);
            track("home_dorder_entry_click", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    public static void customerCenterEvent(String s1, String s2, String s3) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("auto_service_button", s1);
            properties.put("common_problem_types", s2);
            properties.put("bottom_button", s3);
            track("customer_service_center_click", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 找律师模块点击
     *
     * @param s1
     * @param s2
     */
    public static void addFindLawyerEvent(String s1, String s2) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("consulting_entrance", s1);
            properties.put("common_types", s2);
            track("find_lawyer_click", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    public static void relationSensor1(int number_nodes) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("number_nodes", number_nodes);
            track("find_relationship_click", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    public static void relationSensor2() {
        track("Find_relationship_click_details", null);
    }

    public static void lawyerDetailPageMoudle(String module_name) {
        try {
            JSONObject properties = new JSONObject();
            properties.put("module_name", module_name);
            track("click_olawyer_details_page", properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 律师认证进入、下一步、提交
     */
    public static String LAYER_AUTH = "lawyer_certification_enter_button_click";
    public static String LAYER_AUTH_NEXT = "lawyer_certification_next_button_click";
    public static String LAYER_AUTH_SUBMIT = "lawyer_certification_submit_info_button_click";
    //优化券分析和使用
    public static String VIP_COUPON_ANALYZE = "VIP_usage_analysis";
    public static String VIP_COUPON_USAGE = "VIP_coupons_use";

    /**
     * 单个埋点
     *
     * @param eventName   事件名
     * @param actionName  操作名
     * @param actionValue 操作行为
     */
    public static void trackSingleAction(String eventName, String actionName, String actionValue) {
        try {
            JSONObject properties = new JSONObject();
            properties.put(actionName, actionValue);
            track(eventName, properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    /**
     * 多个埋点
     *
     * @param eventName    事件名
     * @param keyAndValues 操作名和操作行为
     */
    public static void trackMultiAction(String eventName, Map<String, Object> keyAndValues) {
        try {
            JSONObject properties = new JSONObject();
            Set<Map.Entry<String, Object>> entrySet = keyAndValues.entrySet();
            for (Map.Entry<String, Object> entry : entrySet) {
                properties.put(entry.getKey(), entry.getValue());
            }
            track(eventName, properties);
        } catch (Exception e) {
            MMLog.e(e.getMessage());
        }
    }

    public static void track(String eventName, JSONObject jsonObject) {
        if (StringUtil.isEmpty(eventName)) {
            return;
        }
        SensorsDataAPI.sharedInstance().track(eventName, jsonObject);
    }
}