package com.winhc.business.utils;

import android.graphics.Color;
import android.support.annotation.DrawableRes;
import android.text.Html;
import android.text.SpannableString;
import android.text.Spanned;
import android.text.style.ForegroundColorSpan;
import android.text.style.RelativeSizeSpan;
import android.widget.TextView;

import com.winhc.widget.util.MMLog;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class TextViewUtil {
    public static void clearTextLeftDrawable(TextView textView) {
        textView.setCompoundDrawablesRelativeWithIntrinsicBounds(null, null, null, null);
    }

    public static void setTextLeftDrawable(TextView textView, @DrawableRes int resId) {
        textView.setCompoundDrawablesRelativeWithIntrinsicBounds(textView.getResources().getDrawable(resId), null, null, null);
    }

    public static void setTextRightDrawable(TextView textView, @DrawableRes int resId) {
        textView.setCompoundDrawablesRelativeWithIntrinsicBounds(textView.getResources().getDrawable(resId), null, null, null);
    }

    public static void setHighString(String s, TextView t, String key) {
        //如果搜索出来的文字跟输入文字有重叠，则改变重叠文字的颜色
        if (s != null && s.contains(key)) {
            int index = s.indexOf(key);
            int len = key.length();
            Spanned temp = Html.fromHtml(s.substring(0, index)
                    + "<font color=#0265D9>"
                    + s.substring(index, index + len) + "</font>"
                    + s.substring(index + len));
            t.setText(temp);
        } else {
            t.setText(s);
        }
    }

    /**
     * @param color   关键字颜色
     * @param text    文本
     * @param keyword 多个关键字
     * @return
     */
    public static SpannableString matcherSearchText(int color, String text, String keyword) {
        SpannableString ss = new SpannableString(text);
        //检测是否含有特殊字符，否则会报错
        String regex = "^[0-9A-Za-z\u4e00-\u9fa5]+$";
        if (keyword.matches(regex)) {
            MMLog.e("字符串不含有特殊符号");
            Pattern pattern = Pattern.compile(keyword);
            Matcher matcher = pattern.matcher(ss);
            while (matcher.find()) {
                int start = matcher.start();
                int end = matcher.end();
                ss.setSpan(new ForegroundColorSpan(color), start, end, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE);
            }
        } else {
            MMLog.e("字符串含有特殊符号");
        }
        return ss;
    }

    public static void setTextView(TextView textView, String source, String key, int color, float scale) {
        if (null == textView || StringUtils.isEmpty(source) || StringUtils.isEmpty(key)) {
            return;
        }
        int startIndex = source.indexOf(key);
        int endIndex = startIndex + key.length();
        setTextView(textView, source, startIndex, endIndex, color, scale);
    }

    public static void setTextView(TextView textView, String source, int startIndex, int endIndex, int color, float scale) {
        if (null == textView || StringUtils.isEmpty(source)) {
            return;
        }
        if (startIndex > endIndex || endIndex > source.length() || startIndex < 0) {
            return;
        }
        SpannableString spannable = new SpannableString(source);
        spannable.setSpan(new ForegroundColorSpan(color == 0 ? Color.BLACK : color), startIndex, endIndex, SpannableString.SPAN_EXCLUSIVE_EXCLUSIVE);
        spannable.setSpan(new RelativeSizeSpan(scale), startIndex, endIndex, SpannableString.SPAN_EXCLUSIVE_EXCLUSIVE);
        textView.setText(spannable);
    }
}
