package com.winhc.http;

import android.text.TextUtils;

import org.json.JSONException;

import java.text.ParseException;

import io.reactivex.SingleObserver;
import io.reactivex.disposables.Disposable;
import retrofit2.HttpException;

/**
 * Create by LZ at 2021/10/12
 */
public abstract class HttpObserver<T> implements SingleObserver<BaseBean<T>> {
    public static final String ERROR_TOKEN = "ERROR_TOKEN";

    @Override
    public void onSubscribe(Disposable d) {

    }

    @Override
    public void onSuccess(BaseBean<T> baseBean) {
        if (null != baseBean) {
            if (baseBean.success()) {
                onResult(baseBean.data());
            } else {
                if (null != baseBean.errorCode && !TextUtils.isEmpty(baseBean.errorCode) && ERROR_TOKEN.equals(baseBean.errorCode)) {
                    onTokenOutDate();
                }
                onFailure(getErrorMessage(new Throwable(baseBean.errorMsg)), baseBean.errorCode);
            }
        } else {
            onFailure("数据异常", "500");
        }
        onHttpEnd();
    }

    @Override
    public void onError(Throwable e) {
        onFailure(getErrorMessage(e), "500");
        onHttpEnd();
    }

    /**
     * 网络请求成功时的处理
     *
     * @param baseBean 该字段可能为空 也可能不为空
     */
    public abstract void onResult(T baseBean);

    public abstract void onFailure(String message, String errorCode);

    /**
     * 网络结束的回调
     */
    public abstract void onHttpEnd();

    private String getErrorMessage(Throwable e) {
        String result = "当前网络不给力，请检查你的网络设置";
        if (e instanceof HttpException && ((HttpException) e).code() == 401) {
            onTokenOutDate();
        } else if (e instanceof JSONException || e instanceof ParseException) {
            result = "系统繁忙，请稍后再试";
        }
        return result;
    }

    /**
     * token过期 需要清除登录信息
     */
    public abstract void onTokenOutDate();
}
