package com.winhc.http;

import java.util.concurrent.ConcurrentHashMap;

import okhttp3.OkHttpClient;
import retrofit2.Retrofit;

public abstract class ServiceFactory {
    private static final ConcurrentHashMap serviceMap = new ConcurrentHashMap();

    /**
     * 需要用户自己构建地址
     *
     * @param clazz
     * @param <T>
     * @return
     */
    public <T> T createService(Class<T> clazz) {
        return createService(clazz, serviceUrl());
    }

    public <T> T createService(Class<T> clazz, String baseUrl) {
        return createService(clazz, baseUrl, buildClient());
    }

    /**
     * 根据类和网址创建服务
     *
     * @param clazz      类
     * @param baseUrl    请求地址
     * @param httpClient 客户端
     * @param <T>        返回的泛型
     * @return
     */
    public <T> T createService(Class<T> clazz, String baseUrl, OkHttpClient httpClient) {
        String classKey = clazz.getSimpleName() + baseUrl;
        Object service = serviceMap.get(classKey);
        if (service == null) {
            Retrofit.Builder builder = buildRetrofit(baseUrl, httpClient);
            service = builder.build().create(clazz);
            serviceMap.put(classKey, service);
        }
        return (T) service;
    }

    public abstract Retrofit.Builder buildRetrofit(String url, OkHttpClient httpClient);

    public abstract OkHttpClient buildClient();

    public abstract String serviceUrl();

    public abstract boolean printLog();
}
